import { DependencyModule, Token, TokenValue } from 'brandi';
import { ScopeSyntax } from 'brandi/lib/typings/container/syntax/ScopeSyntax';
import { TypeSyntax } from 'brandi/lib/typings/container/syntax/TypeSyntax';
import { UnknownCreator, UnknownCreatorParameters } from 'brandi/lib/typings/types';
import * as R from 'fp-ts/Reader';
/**
 * Type utils
 */
type FilterFlags<Base, Condition> = {
    [Key in keyof Base]: Base[Key] extends Condition ? Key : never;
};
type ExtractCond<Base, Condition> = Pick<Base, FilterFlags<Base, Condition>[keyof Base]>;
export interface DependencyModuleWithTokens<TOKENS extends Record<string, Token>> extends DependencyModule {
    readonly tokens: TOKENS;
    readonly allTokens: Token[];
}
type Bindings<out TOKENS> = {
    [K in keyof TOKENS]: TOKENS[K] extends Token<infer V> ? (arg: TypeSyntax<V>) => ScopeSyntax | [ScopeSyntax, keyof typeof Scope] | void : never;
};
type Module<TOKENS extends Record<string, Token>> = DependencyModuleWithTokens<TOKENS>;
declare enum Scope {
    CONTAINER = "CONTAINER",
    RESOLUTION = "RESOLUTION",
    TRANSIENT = "TRANSIENT",
    SINGLETON = "SINGLETON"
}
declare const makeInjector: <TOKENS extends Record<string, Token<unknown>>>(registry: Map<UnknownCreator, R.Reader<Module<TOKENS>, TokenValue<unknown>[]>>) => <T extends UnknownCreator>(target: T, ...tokensKeys: (UnknownCreatorParameters<T> extends infer T_1 extends unknown[] ? { [K in keyof T_1]-?: ExtractCond<TOKENS, Token<UnknownCreatorParameters<T>[K]>> extends Record<infer I extends string | number | symbol, TokenValue<unknown>> ? I : never; } : never) extends (keyof TOKENS)[] ? (keyof TOKENS)[] & (UnknownCreatorParameters<T> extends infer T_1 extends unknown[] ? { [K in keyof T_1]-?: ExtractCond<TOKENS, Token<UnknownCreatorParameters<T>[K]>> extends Record<infer I extends string | number | symbol, TokenValue<unknown>> ? I : never; } : never) : never) => T;
type Injector<TOKENS extends Record<string, Token>> = ReturnType<typeof makeInjector<TOKENS>>;
declare class ContravariantMaker<TOKENS extends Record<string, Token>, in BINDINGTOKENS extends keyof TOKENS> {
    private maker;
    constructor(make: R.Reader<Bindings<Pick<TOKENS, BINDINGTOKENS>>, Module<TOKENS>>);
    make(bindings: Bindings<Pick<TOKENS, BINDINGTOKENS>>): Module<TOKENS>;
}
type MeetBindings<M1, M2> = M1 extends DIModule<infer T1, infer B1> ? M2 extends DIModule<infer T2, infer B2> ? B2 extends keyof T1 ? T1 extends T2 ? Exclude<B1, keyof T2> | B2 : T2 extends T1 ? Exclude<B2, keyof T1> | B1 : Exclude<B1 | B2, B1 & B2> : B1 | B2 : never : never;
export declare class DIModule<TOKENS extends Record<string, Token>, in BINDINGTOKENS extends keyof TOKENS> extends ContravariantMaker<TOKENS, BINDINGTOKENS> {
    readonly injector: Injector<TOKENS>;
    constructor(injector: Injector<TOKENS>, make: R.Reader<Bindings<Pick<TOKENS, BINDINGTOKENS>>, Module<TOKENS>>);
    bind<Bound extends BINDINGTOKENS>(key: Bound, binding: Bindings<TOKENS>[Bound]): DIModule<TOKENS, Exclude<BINDINGTOKENS, Bound>>;
    combine<TOKENS2 extends Record<string, Token>, BINDINGTOKENS2 extends keyof TOKENS2>(mod2: DIModule<TOKENS2, BINDINGTOKENS2>): DIModule<TOKENS & TOKENS2, MeetBindings<DIModule<TOKENS, BINDINGTOKENS>, DIModule<TOKENS2, BINDINGTOKENS2>>>;
}
export declare const makeInjectedModule: <TOKENS extends Record<string, Token<unknown>>>(tokens: TOKENS) => DIModule<TOKENS, keyof TOKENS>;
export {};
